local cv_cameraroll = CV_RegisterVar{"cameraroll", "On", CV_SAVE|CV_CALL|CV_NOINIT|CV_SHOWMODIF, CV_OnOff,
	do
		if not maptol then
			print("Please be ingame for the change to take effect.")
		end
	end
}
print('Registered console variable "cameraroll"')

local cv_cam_rollspeed = CV_RegisterVar{"cam_rollspeed", "0.1", CV_FLOAT|CV_SAVE|CV_SHOWMODIF, {MIN=0, MAX=FRACUNIT}}
print('Registered console variable "cam_rollspeed"')

CV_RegisterVar{"cv_", "on", CV_NETVAR, CV_OnOff, func}

// Centers you!
local function resetRollAngle(rollangle)
	// The speed at which you are centered at.
	local speed = FixedAngle(6*FRACUNIT)

	if rollangle > FixedAngle(0) + speed then // rollangle more than centered + centering speed?
		rollangle = $ - speed // Subtract the speed!
	elseif rollangle < FixedAngle(0) - speed then // rollangle less than centered - centering speed?
		rollangle = $ + speed // Add the speed!
	else // rollangle between +centering speed and -centering speed?
		rollangle = FixedAngle(0) // Center!
	end

	return rollangle // Return the resulting rollangle.
end

local function getSlopeAngleForRollAngle(mo, dontflip)
	local lookAngle = R_PointToAngle(mo.x, mo.y) // Get the angle that we're looking from.

	if not R_PointToDist(mo.x, mo.y) then // Camera at player?
		lookAngle = mo.angle // Use player angle instead of whatever R_PointToAngle generates..
	end

	// zangle and direction of the slope we're on (if any).
	local zangle = mo.standingslope.zangle
	local xydirection = mo.standingslope.xydirection

	if mo.eflags & MFE_VERTICALFLIP and not dontflip then // Flipped?
		zangle = InvAngle(zangle) // Flip the zangle too!
	end

	// Fancy angle math that rolls the sprite against the slope.
	// Trust me on this one.
	return FixedMul(zangle, sin(lookAngle - xydirection))
end

local function normalPlayerStuff(mo)
	// Generic sanity checks.
	if not mo.subsector or not mo.subsector.valid
	or not mo.subsector.sector or not mo.subsector.sector.valid then
		return
	end

	local player = mo.player // Get player
	local sector = mo.subsector.sector // Get sector

	if not P_IsObjectOnGround(mo) then // Off the ground?
		if not player.sloperollangle_override then
			mo.rollangle = resetRollAngle(mo.rollangle) // Center that rollangle!
		end

		if cv_cameraroll.value and player.viewrollangle ~= nil then // They've enabled the camera rolling?
			player.viewrollangle = FixedMul($, FRACUNIT - cv_cam_rollspeed.value) // Smoothly reset it to 0.
		end

		return // Don't continue.
	end

	if player.pflags & PF_SPINNING and mo.standingslope then // Spinning?
		if not player.sloperollangle_override then
			mo.rollangle = FixedAngle(0) // Reset rollangle.
		end

		if cv_cameraroll.value and player.viewrollangle ~= nil then // They've enabled the camera rolling?
			// Instead of resetting,
			// calculate the normal camera rolling.
			// Spinning shouldn't effect the camera roll!
			player.viewrollangle = $ - FixedMul(getSlopeAngleForRollAngle(mo, true) + $, cv_cam_rollspeed.value)
		end

		return // Don't continue.
	elseif not mo.standingslope then // No slope?
		if not player.sloperollangle_override then
			mo.rollangle = FixedAngle(0) // Reset rollangle.
		end

		if cv_cameraroll.value and player.viewrollangle ~= nil then // They've enabled the camera rolling?
			// Adjust camera for maximum motion sickness.
			player.viewrollangle = FixedMul($, FRACUNIT - cv_cam_rollspeed.value) // Smoothly reset it to 0.
		end

		return // Don't continue.
	end

	// Use fancy formula that sets the player's roll angle to what it should be.
	if not player.sloperollangle_override then
		mo.rollangle = getSlopeAngleForRollAngle(mo)
	end

	if cv_cameraroll.value and player.viewrollangle ~= nil then // They've enabled the camera rolling?
		// Adjust camera for maximum motion sickness.
		player.viewrollangle = $ - FixedMul(getSlopeAngleForRollAngle(mo, true) + $, cv_cam_rollspeed.value)
	end
end

local function mobjThinker(mo)
	// Generic sanity checks.
	if not mo.player or not mo.player.valid then
		return
	end

	local player = mo.player

	if cv_cameraroll.value and player.viewrollangle == nil and leveltime > 0 then // Viewroll on, nil viewroll, and printable?
		// Uh oh, we got a problem!
		print(
			'\x80You do not have an EXE that supports view rolling!' .. '\n'
			.. 'Please download ' .. '\x83' .. 'Fickle\'s viewroll EXE\x80, or ' .. '\x83' .. 'a newer version of SRB2\x80.'
		)
		COM_BufInsertText(player, "cameraroll 0") // Reset cameraroll so we don't get the error again.
	elseif not cv_cameraroll.value and player.viewrollangle ~= nil then // Off, but viewroll exists?
		player.viewrollangle = FixedMul($, FRACUNIT - cv_cam_rollspeed.value) // Smoothly reset it to 0.
	end

	if not (maptol & TOL_NIGHTS) then // NiGHTS?
		normalPlayerStuff(mo)
	end
end

// Add le hooks.
addHook("MobjThinker", mobjThinker, MT_PLAYER)

addHook("ThinkFrame", function()
    for player in players.iterate
	if player.mo.skin == "henchman" and
            player.spinitem
            player.spinitem = 0
        end
    end
end)

addHook("ThinkFrame", do
for player in players.iterate do
if player.mo and player.mo.valid 
if player.mo.skin ~= "henchman" return end
if player.pflags & PF_JUMPED or player.pflags & PF_THOKKED or player.pflags & PF_STASIS or player.pflags & PF_JUMPSTASIS return end
if (player.panim == PA_IDLE or player.panim == PA_WALK or player.panim == PA_RUN or player.panim == PA_DASH)
and not P_IsObjectOnGround(player.mo)
and player.mo.momz < 1
and not player.powers[pw_carry]
player.mo.state = S_PLAY_FALL
end
if (player.panim == PA_IDLE or player.panim == PA_WALK or player.panim == PA_RUN or player.panim == PA_DASH)
and not P_IsObjectOnGround(player.mo)
and player.mo.momz > 0
and not player.powers[pw_carry]
player.mo.state = S_PLAY_SPRING
end
end
end
end)

addHook("ThinkFrame", function()
    for player in players.iterate do
	 --local player = mo.player
        if not (player and player.valid and player.mo.skin == "henchman") then break end -- player does not exist, don't run
        if not (player.realmo and player.realmo.valid) then break end -- player does not exist 2: revengence, don't run
		
		if (player.pflags & PF_SPINNING and player.speed > 15*FRACUNIT)
		or player.charability == CA_THOK and (player.pflags & PF_THOKKED and player.speed > 15*FRACUNIT) and not (player.pflags & PF_SHIELDABILITY)

            local traily = P_SpawnGhostMobj(player.mo)
            traily.fuse = 2
			traily.colorized = false
			traily.blendmode = AST_ADD
            traily.rollangle = player.mo.rollangle
            if (traily.tracer) then
                traily.tracer.fuse = 2
            end
        end
	end
end)

local function thicc(p)
	if p.jt == nil then
		p.jt = 0
		p.jp = false
		p.sp = false
		p.tk = false
		p.tr = false
	end
	if p.jt > 0 then
		p.jt = p.jt - 1
	end
	if p.jt < 0 then
		p.jt = p.jt + 1
	end
	if p.mo.momz < 1 then
		p.jp = false
	end
	if p.mo.state != S_PLAY_CLIMB and p.mo.eflags != p.mo.eflags | MFE_GOOWATER then
		if p.mo.momz > 0 and p.jp == false and p.mo.state != S_PLAY_FLY and p.mo.state != S_PLAY_SWIM and p.mo.state != S_PLAY_FLY_TIRED and p.mo.state != S_PLAY_WALK and p.mo.state != S_PLAY_RUN and p.mo.state != S_PLAY_WALK and p.mo.state != S_PLAY_BOUNCE_LANDING then
			p.jp = true
			p.jt = 5
		end
		if p.mo.momz > 0 and p.jt < 0 and p.mo.state != S_PLAY_FLY and p.mo.state != S_PLAY_SWIM and p.mo.state != S_PLAY_FLY_TIRED and p.mo.state != S_PLAY_WALK and p.mo.state != S_PLAY_RUN and p.mo.state != S_PLAY_WALK and p.mo.state != S_PLAY_BOUNCE_LANDING then
			p.jp = true
			p.jt = 5
		end
	elseif p.mo.eflags == p.mo.eflags | MFE_GOOWATER then
		p.jp = true
	end
	if p.mo.state == S_PLAY_BOUNCE_LANDING then
		p.jt = -5
	end
	/*
	if p.pflags != p.pflags | PF_SPINNING or p.pflags == p.pflags | PF_JUMPED then
		p.sp = false
	end
	if p.pflags == p.pflags | PF_SPINNING and p.pflags != p.pflags | PF_JUMPED and p.sp == false and p.jt < 1 then
		p.sp = true
		p.jt = -5
	end
	*/
	if p.pflags != p.pflags | PF_THOKKED then
		p.tk = false
	end
	if p.pflags == p.pflags | PF_THOKKED and p.tk == false then
		p.tk = true
		p.jt = 5
	end
	if p.mo.state != S_PLAY_FLY_TIRED then
		p.tr = false
	end
	if p.mo.state == S_PLAY_FLY_TIRED and p.tr == false then
		p.tr = true
		p.jt = 5
	end

	-- Check if the player character's skin is "henchman"
	if p.mo.skin == "henchman" then
		p.maths = p.jt * FRACUNIT
		p.maths = p.maths / 10
		p.mo.spriteyscale = p.maths + FRACUNIT
		
		p.maths = p.jt * p.spinheight
		if p.mo.state != S_PLAY_ROLL then
			p.maths = p.jt * p.height
		end
		p.maths = p.maths / 20
		p.mo.spriteyoffset = -1 * p.maths
		
		p.maths = p.jt * FRACUNIT
		p.maths = p.maths / 10
		p.maths = p.maths * -1
		p.mo.spritexscale = p.maths + FRACUNIT
	end
end

addHook("PlayerThink", thicc)
local function tailsthink(mo)
	mo.spriteyscale = mo.tracer.spriteyscale
	mo.spritexscale = mo.tracer.spritexscale
	mo.spriteyoffset = mo.tracer.spriteyoffset
end
addHook("MobjThinker", tailsthink, MT_TAILSOVERLAY)

addHook("PreThinkFrame", function()
	for player in players.iterate do										//You gotta be a player
		if player.fakeroll == nil then										//If player.fakeroll doesn't exist then
			player.fakeroll = 0										//make it exist
		end
		player.mo.rollangle = (player.mo.rollangle - player.fakeroll)
	end
end)

addHook("ThinkFrame", function()
	for player in players.iterate do										//you gotta be a player
		if P_IsObjectOnGround(player.mo) and player.mo.skin == "henchman" and player.speed > 2000000 then					//you gotta be running on the ground
			player.fakeroll = ((player.mo.angle - R_PointToAngle(player.mo.x, player.mo.y)) / 2)		//The actual angle script
			if player.fakeroll > 300000000 then								//if you're going too far to the left then
				player.fakeroll = 300000000								//cap your angle
			end	
			if player.fakeroll < -300000000 then								//if you're going too far to the right then
				player.fakeroll = -300000000								//cap your angle
			end
		else													//if you're not running or you're not on the ground then
			player.fakeroll = 0										//become vertical again
		end
	end
end)

addHook("PostThinkFrame", function()
	for player in players.iterate do										//You gotta be a player
	player.mo.rollangle = (player.mo.rollangle + player.fakeroll)
	end
end)

//Enable the code below to draw some random crap (for debug reasons only)
//hud.add(function(v)
//	for player in players.iterate do
//		v.drawNum(160, 5, player.mo.rollangle, V_ORANGEMAP|V_30TRANS)
//		v.drawNum(160, 15, player.mo.angle, V_ORANGEMAP|V_30TRANS)
//		v.drawNum(160, 25, R_PointToAngle(player.mo.x, player.mo.y), V_ORANGEMAP|V_30TRANS)
//		v.drawNum(160, 35, player.speed, V_ORANGEMAP|V_30TRANS)
//	end
//end)

local function RefreshAbility(player)
	player.pflags = $ | PF_JUMPED & ~PF_THOKKED
	player.secondjump = 0;
end

local function PreventAirWalk(player)
    if (player.mo.state == S_PLAY_WALK or player.mo.state == S_PLAY_RUN)
    and not player.powers[pw_carry]
        player.mo.state = S_PLAY_FALL
    end
end

local function SpinCharThink(player)
    -- Ignore if character is grounded
    if (P_IsObjectOnGround(player.mo))
    player.mo.ao_donecheck = false
    return
end

    PreventAirWalk(player)

    -- If spinning in the air
    if (player.pflags & PF_SPINNING)
        -- Turn off the spinning flag; this means you won't turn slowly like when rolling on ground
        player.pflags = $ & ~PF_SPINNING
        -- Use the jump state instead
        player.mo.state = S_PLAY_JUMP
        -- Enable air ability
        player.mo.ao_donecheck = true
    end

    if (player.mo.ao_donecheck and not player.mo.coyoteTime)
        player.mo.ao_donecheck = false
        RefreshAbility(player)
    end
end

local function NonSpinCharThink(player)
    -- Ignore if character is grounded
    if (P_IsObjectOnGround(player.mo))
        player.mo.ao_donecheck = false
        return
    end
    
    PreventAirWalk(player)
    
    if (player.mo.state == S_PLAY_SPRING)
        player.mo.ao_donecheck = false
    end

    if not (player.mo.ao_donecheck or player.mo.coyoteTime)
        player.mo.ao_donecheck = true
        RefreshAbility(player)
    end
end

local function AirOptionsThink(player)
    -- Run spin or nonspin character logic
    if (player.charflags & SF_NOJUMPSPIN)
        NonSpinCharThink(player)
    else
        SpinCharThink(player)
    end

end

addHook("PreThinkFrame", function()
	for player in players.iterate do
		AirOptionsThink(player);
	end
end)

--Xian
--Kirb the Korb

-- Spin Dash turning modification
addHook("PlayerThink", function(player)
	if not (player.mo and player.mo.valid and player.mo.skin == "henchman" and player.playerstate == PST_LIVE) then return end
	
	--Apply only to players with the normal spindash
	if player.charability2 ~= CA2_SPINDASH then
		return
	end

	if (player.pflags & PF_SPINNING)									-- Check to see if spinning
	and not (player.pflags & PF_STARTDASH) 								-- Revving the Spindash does not count
	and (P_IsObjectOnGround(player.mo))									-- Drifting only occures when on the ground
	and (player.cmd.forwardmove ~= 0) and (player.cmd.sidemove ~= 0)	-- only turn when the player wants to							
	and (player.speed > 1*FRACUNIT) then								-- minimum speed requirment


		-- Current information about the player
		local mom = FixedHypot(player.mo.momx, player.mo.momy)					-- Base Speed (No friction or drag)
		local motion = R_PointToAngle2(0, 0, player.mo.momx, player.mo.momy) 	-- base direction (No outside forces)

		-- Direction player is inputting
		local pangle = player.cmd.angleturn<<16 + R_PointToAngle2(0, 0, player.cmd.forwardmove*FRACUNIT, -player.cmd.sidemove*FRACUNIT)

		-- Calculate the new direction 
		local turn = 0
		local turning = FixedAngle(FixedMul(FixedDiv(AngleFixed(ANG2), 29*FRACUNIT/30), player.mo.friction)) -- scale by friction
		local dif = pangle - motion
		if dif > turning or dif < -turning then	-- Presision handling
			if dif > 0 then						-- Angle between 0 and 180 degrees (Left)
				turn = motion + turning	-- Turn Left
			else
				turn = motion - turning	-- Turn Right
			end
		else
			turn = pangle				--Go straight
		end

		P_InstaThrust(player.mo, turn, mom) -- Apply new movement and replace old movement
	end
end)

-- Scale dust made by the spindash to match how charged it is
local SpindustCheck = false
addHook("SpinSpecial", function(player)

	if not (skins[player.mo.skin].flags & SF_NOSPINDASHDUST)	-- Player's skin didn't already override the dust
	and (player.charability2 == CA2_SPINDASH)					-- Player's skin uses the Spindash
	and (player.pflags & PF_STARTDASH) 							-- Player is charging the spindash
	and (P_IsObjectOnGround(player.mo)) then					-- Player is on the ground

		player.charflags = $ | SF_NOSPINDASHDUST	-- Disable original spindash dust

		-- Check if player is in a liquid so that the spindash makes bubbles instead of dust
		local particleType
		if (player.mo.eflags & (MFE_TOUCHWATER | MFE_UNDERWATER)) then
			particleType = MT_SMALLBUBBLE
		else
			particleType = MT_SPINDUST
		end

		local charge = FixedDiv(player.dashspeed, player.maxdash)	-- Get a 0 - 1 percentage charge of the spindash

		if (P_RandomFixed() < charge + FU/2) then
			-- create dust particle and set its effects/scale/velocity based off of how charged the spindash is
			local dust = P_SpawnMobjFromMobj(player.realmo, P_RandomRange(-2, 2)*FU, P_RandomRange(-2, 2)*FU, 0, particleType)
			dust.momx = P_RandomRange(-2, 2) * charge
			dust.momy = P_RandomRange(-2, 2) * charge
			dust.momz = P_RandomRange( 0, 2) * charge
			P_Thrust(dust, player.mo.angle, -12 * charge)
			dust.destscale = 3 * charge / 2
		end

	end

	-- Change nothing else
	return false
end)

local function PreThinkJump(player)
	P_DoJump(player, true);

	-- Prevent using abilities immediately, by removing jump inputs.
	player.cmd.buttons = $1 & ~BT_JUMP;
	player.pflags = $1 | (PF_JUMPDOWN|PF_JUMPSTASIS);
end

local function RestoreAbility(player)
	player.secondjump = 0;
	player.pflags = $1 & ~PF_THOKKED;
end

local function RemoveAbility(player)
	player.secondjump = UINT8_MAX;
	player.pflags = $1 | PF_THOKKED;
end

local function RecoveryThink(player, mo, pressedJump, latency)
	mo.coyoteTime = 0; -- Reset coyote time
	mo.recoveryWait = $1 + 1; -- Increment recovery wait time.

	-- The recovery jump from SA2, where you can jump out of your pain state.
	if (pressedJump == true)
		local painThrust = 69*FRACUNIT/10;
		local baseGravity = max(1, abs(P_GetMobjGravity(mo)));
		local painTime = FixedFloor(FixedDiv(painThrust, baseGravity)) >> FRACBITS;

		local baseRecoveryWait = (painTime * 1); -- Double the length of your pain state, - your latency.

		if (mo.recoveryWait > baseRecoveryWait - latency)
			PreThinkJump(player);

			--[[
			RemoveAbility(player);

			-- Reset momentum so you can move a bit.
			player.mo.momx = 0;
			player.mo.momy = 0;
			--]]

			RestoreAbility(player);
		end
	end
end

local function CoyoteThink(player, mo, pressedJump, latency)
	-- "Coyote time" is how much time you have after leaving the ground where you can jump off.
	-- Many modern platformers do this, especially 3D.
	-- Prevents lots of "the jump didn't jump".
	local baseCoyoteTime = TICRATE/4; -- 0.25 seconds, + your latency.

	-- Check if you're in a state where you would normally be allowed to jump.
	local canJump = false;
	if ((P_IsObjectOnGround(mo) == true) or (P_InQuicksand(mo) == true))
	and (player.powers[pw_carry] == CR_NONE)
		canJump = true;
	end

	if (player.pflags & PF_JUMPED) or (player.powers[pw_justsprung])
		-- We jumped. We should not have coyote time.
		mo.coyoteTime = 0;
	elseif (canJump == true)
		-- Set the coyote time while in a state where you can jump.
		mo.coyoteTime = baseCoyoteTime + latency;
	else
		if (pressedJump == true) and (mo.coyoteTime > 0)
			-- Pressed jump in a state where you can't jump,
			-- but you have coyote time. So we'll give you a jump anyway!
			PreThinkJump(player);
			RestoreAbility(player);
			mo.coyoteTime = 0;
		end

		if (mo.coyoteTime > 0)
			-- Reduce coyote timer while in a state where you can't jump.
			mo.coyoteTime = $1 - 1;
		end
	end
end

local function JumpLeniencyThink(player)
	if not (player.mo and player.mo.valid) or (player.mo.skin ~= "henchman")
		return;
	end
	local mo = player.mo;

	if (player.playerstate != PST_LIVE)
		return;
	end

	local latency = player.cmd.latency;

	-- Init variables
	if (mo.coyoteTime == nil)
		mo.coyoteTime = 0;
	end

	if (mo.recoveryWait == nil)
		mo.recoveryWait = 0;
	end

	if (player.exiting)
	or (player.pflags & PF_JUMPSTASIS)
	or ((player.powers[pw_nocontrol] > 0) and not (player.powers[pw_nocontrol] & (1<<15)))
		-- Can't control anyway.
		return;
	end

	local pressedJump = false;
	if (player.cmd.buttons & BT_JUMP) and not (player.pflags & PF_JUMPDOWN)
		pressedJump = true;
	end

	if (P_PlayerInPain(player) == true)
		RecoveryThink(player, mo, pressedJump, latency);
		return;
	else
		-- Reset recovery wait time outside of pain state.
		mo.recoveryWait = 0;
	end

	CoyoteThink(player, mo, pressedJump, latency);
end

addHook("PreThinkFrame", function()
	if (mapheaderinfo[gamemap].jumpleniency)
		-- Don't run more than once on maps that have their own (SUGOI)
		return;
	end

	for player in players.iterate do
		JumpLeniencyThink(player);
	end
end);

addHook("ThinkFrame", do
    for player in players.iterate
	   if (player.mo and player.mo.skin == "henchman")

	   if not (player.mo.usedown) then
	   player.mo.usedown = 0
	   end
	   
	   if not (player.mo.flydrop) then
	   player.mo.flydrop = 0
	   end
	  
	   if (player.cmd.buttons & BT_TOSSFLAG)
	   and (player.mo.usedown == 0)
	   and not P_IsObjectOnGround(player.mo)
	   player.mo.usedown = 1
	   player.mo.flydrop = 0
	   player.mo.state = S_PLAY_JUMP
	   player.pflags = $1|PF_JUMPED
	   end
	   
	   if not (player.cmd.buttons & BT_TOSSFLAG)
	   and (player.mo.usedown == 1)
	   player.mo.usedown = 0
	   end
	   
	    end
	end
end)

addHook("PlayerCanDamage", function(player, mobj)
	if player.mo and player.mo.skin == "henchman"
	and player.mo.state == S_PLAY_ROLL
	and (mobj.flags&MF_ENEMY or mobj.flags&MF_BOSS or mobj.flags&MF_MONITOR) return true end
end, MT_PLAYER)

//Bomb Throw Ability - By Simon_T
//This is free to be used in other people's mods
//But it would be great if you credit me as the original creator ;)

addHook("ThinkFrame", do
	for player in players.iterate
		if player.mo and player.mo.skin == "henchman" //If you're Fang
			if not player.exiting //If you're leaving a level, you won't need them, right?
				if not (P_PlayerInPain(player) or (player.playerstate == PST_DEAD)) //Hurt/dead weasels can't throw bombs
					if ((player.cmd.buttons & BT_CUSTOM3) and not (player.weapondelay)) //If you press Toss Flag...
						S_StartSound(player.mo, sfx_s3k51) //...Play the bomb throwing sound...
						local bomb = P_SpawnMobj(player.mo.x, player.mo.y, player.mo.z + player.mo.height, MT_FBOMB) //...And spawn the bomb itself
						bomb.target = player.mo //The bomb can't hurt its user
						if (player.mo.momz > 0) //If you're jumping/springing/bouncing...
							bomb.momz = (player.mo.momz)+(15/FRACUNIT) //...The bomb gets some height boost
						else //But if you're falling...
							bomb.momz = 10/FRACUNIT //...The bomb just flies at this fixed height
						end
						P_InstaThrust(bomb, player.mo.angle, ((player.speed)+(10*FRACUNIT))) //The bomb describes an arc while flying, affected by your current speed
						if (player.powers[pw_super]) //If you're Super Fang...
							player.weapondelay = TICRATE*1 //...You can throw bombs 3X faster
						else //But if that's not the case...
							player.weapondelay = TICRATE*1 //...You have to wait three seconds between bombs
						end
					end
				end
			end
		end
	end
end)

-- toggle variable
local holdflymode = true

-- hold fly mode
addHook('JumpSpecial', function(player)
	if player.mo and player.mo.valid
		-- check the situation is valid
		if (holdflymode) and (player.cmd.buttons & BT_JUMP)
			-- tell the game to propel the player into the air in flight state.
			player.fly1 = 1
		end
	end
end)

local enableHoldFly = function(player, arg1)
	-- toggle
	if arg1 == nil then
		holdflymode = not holdflymode
		local onoffprint = "off"
		if (holdflymode) then onoffprint = "on" end
		CONS_Printf(player, "hold fly "..onoffprint..".")
		return
	end

	-- turn on
	if arg1:lower() == "on" or arg1:lower() == "yes" or arg1 == "1" or arg1:lower() == "true" then
		holdflymode = true
		CONS_Printf(player, "hold fly on.")
		return
	end
	
	-- turn off
	if arg1:lower() == "off" or arg1:lower() == "no" or arg1 == "0" or arg1:lower() == "false" then
		holdflymode = false
		CONS_Printf(player, "hold fly off.")
		return
	end

	-- invalid
	CONS_Printf(player, "Invalid argument specified. use 'holdfly' to toggle, or use on/yes/1/true or off/no/0/false.")
	return
end

-- hook up command
COM_AddCommand("holdfly", enableHoldFly, 0)